\\ Copyright 2015, 2016, 2017, 2018, 2019 Kevin Ryde
\\
\\ This file is free software; you can redistribute it and/or modify it
\\ under the terms of the GNU General Public License as published by the Free
\\ Software Foundation; either version 3, or (at your option) any later
\\ version.
\\
\\ This file is distributed in the hope that it will be useful, but
\\ WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
\\ or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
\\ for more details.
\\
\\ You should have received a copy of the GNU General Public License along
\\ with this file.  See the file COPYING.  If not, see
\\ <http://www.gnu.org/licenses/>.


default(recover,0);
default(strictargs,1);
read("geom.gp");
read("geom-inertia.gp");
read("geom-wip.gp");
read("test-helpers.gp");

option_verbose = 0;

\\ 1 if sign() works on t_QUAD, which it does in GP 2.9 up
test_have_sign_of_quad = iferr(sign(quadgen(12));1, e, 0);
print("test_have_sign_of_quad "test_have_sign_of_quad);


\\-----------------------------------------------------------------------------
\\ geom_projection()

check_equal(geom_projection(I, 9), 0, "geom_projection()");
check_equal(geom_projection(I, 9+9*I), 1/2+1/2*I);


\\-----------------------------------------------------------------------------
\\ geom_turn_LSR()

check_equal(geom_turn_LSR(1,2, 3), 0, "geom_turn_LSR()");
check_equal(geom_turn_LSR(1,2, -3), 0);
check_equal(geom_turn_LSR(1,2, 3+I), 1);
check_equal(geom_turn_LSR(1,2, 3-I), -1);
check_equal(geom_turn_LSR(I,2*I, -1), 1);
check_equal(geom_turn_LSR(I,2*I, 1), -1);
\\ {
\\   check_equal(geom_turn_LSR([0,0],
\\                             [geom_sqrt_as_quad(3),geom_sqrt_as_quad(5)],
\\                             [geom_sqrt_as_quad(7),geom_sqrt_as_quad(11)]), 1);
\\ }

test_random_range(lo,hi) = lo+random(hi-lo+1);

test_turn_xydir(p1,p2,p3) = \
  geom_point_rotate_xydir(p3-p1, geom_conj(p2-p1));

test_turn_LSR_by_xydir(p1,p2,p3) =
{
  my(y=geom_point_y(test_turn_xydir(p1,p2,p3)));
  if(test_have_sign_of_quad,
     sign(y),
     sign(y*1.0));
}
{
  for(rep=1,50,
     my(p1=[test_random_range(-20,20),test_random_range(-20,20)],
        p2=[test_random_range(-20,20),test_random_range(-20,20)],
        p3=[test_random_range(-20,20),test_random_range(-20,20)]);
     check_equal(geom_turn_LSR(p1,p2,p3),
                 test_turn_LSR_by_xydir(p1,p2,p3),
                 "geom_turn_LSR() vs test_turn_LSR_by_xydir()"));
}


\\-----------------------------------------------------------------------------
\\ geom_is_point_colinear()

check_equal(geom_is_point_colinear(0,0), 1, "geom_is_point_colinear()");
check_equal(geom_is_point_colinear(1,2), 1, "geom_is_point_colinear() 1,2");
check_equal(geom_is_point_colinear(1,I), 0, "geom_is_point_colinear() 1,I");
check_equal(geom_is_point_colinear([1,2,3],[-1,-2,-3]), 1);
check_equal(geom_is_point_colinear([1,2,3],[-1,-2,-4]), 0);
check_equal(geom_is_point_colinear([-1,2,3],[-1,-2,-3]), 0);
check_equal(geom_is_point_colinear([1,2,3],[-1,-2]), 0);

test_is_point_colinear_by_abs(p1,p2) =
{
  my(s1=p1*geom_abs(p2), s2=p2*geom_abs(p1));
  nearly_equal(s1,s2) || nearly_equal(s1,-s2);
}
{
  forvec(p1=[[-1,1],[-1,1],[-1,1]],
        forvec(p2=[[-1,1],[-1,1],[-1,1]],
              my(want=p1==p2 || p1==-p2 || p1==[0,0,0] || p2==[0,0,0]);
              check_equal(geom_is_point_colinear(p1,p2), want,
                          Str("geom_is_point_colinear() units "p1" "p2));
              check_equal(test_is_point_colinear_by_abs(p1,p2), want,
                          Str("test_is_point_colinear_by_abs() units "p1" "p2))));
}


\\-----------------------------------------------------------------------------
\\ geom_is_point_on_line()

check_equal(geom_is_point_on_line(0, -1,1), 1, "geom_is_point_on_line()");
check_equal(geom_is_point_on_line(I, -1,1), 0);
check_equal(geom_is_point_on_line(2, 0,1), 1, "geom_is_point_on_line() 2");
check_equal(geom_is_point_on_line(1+I, -1-I, 5+5*I), 1);


\\-----------------------------------------------------------------------------
\\ geom_inertia_scale_line()

{
  my(M1=geom_inertia_of_segment(-1,1),
     M2=geom_inertia_of_segment(-2,2));
  check_equal(geom_inertia_scale_line(M1,2), M2,
              "geom_inertia_scale_line() segment");
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_scale_area()

{
  my(M1=geom_inertia_of_triangle(1,I),
     M2=geom_inertia_of_triangle(2,2*I));
  check_equal(geom_inertia_scale_area(M1,2), M2,
              "geom_inertia_scale_area() triangle");
}


\\-----------------------------------------------------------------------------
\\ geom_inertia_of_intformal_line()

test_inertia_of_segment_by_intformal(p1,p2) =
{
  geom_inertia_of_intformal_line(0,1,
                                 geom_point_x(p1) + 't*(geom_point_x(p2)-geom_point_x(p1)),
                                 geom_point_y(p1) + 't*(geom_point_y(p2)-geom_point_y(p1)),
                                 geom_point_z(p1) + 't*(geom_point_z(p2)-geom_point_z(p1)));
}
check_equal(test_inertia_of_segment_by_intformal([0,0,0], [2,0,0]), \
            geom_inertia_of_segment_m1([0,0,0], [2,0,0]));
check_equal(test_inertia_of_segment_by_intformal([1,2,3], [5,7,20]), \
            geom_inertia_of_segment_m1([1,2,3], [5,7,20]));

\\ (x/a)^2 + (y/b)^2 = 1
\\ y/b = sqrt(1 - (x/a)^2)
\\ y = b*sqrt(1 - (x/a)^2)
test_inertia_of_flat_wedge_by_intformal(l) = \
  geom_inertia_of_intformal_line(0,l, 't,0,0, l-'t) / (1/2*l^2);

check_equal(test_inertia_of_flat_wedge_by_intformal(3), \
            geom_inertia_of_triangle_m1(0,0, 3));
check_equal(test_inertia_of_flat_wedge_by_intformal(13), \
            geom_inertia_of_triangle_m1(0,0, 13));


\\------------------------------------------------------------------------------

\\ (x/a)^2 + (y/b)^2 = 1
\\ y/b = sqrt(1 - (x/a)^2)
\\ y = b*sqrt(1 - (x/a)^2)
\\ Pol(sqrt(1+x^2 + O(x^20)))

\\ density as function sqrt(1-t^2), and divide by total mass Pi/2 that gives
test_inertia_of_flat_ellipse_by_intformal(a) =
{
  a^2/(Pi/2)
  * geom_inertia_of_intformal_line(-1,1, 't,0,0,
                                   Pol(sqrt(1-'t^2 + O('t^200)),'t));
}
check_nearly(test_inertia_of_flat_ellipse_by_intformal(1), \
             geom_inertia_of_ellipse_m1(1,0), \
             "", 1e-2)


\\-----------------------------------------------------------------------------
\\ geom_complex_X_intercept()
\\ geom_complex_Y_intercept()

check_equal(geom_complex_X_intercept(19+3*I,17+2*I), 13)


\\-----------------------------------------------------------------------------

print("test-wip.gp end, ",check_count," tests");
quit(0);
